// Marin Alexandru Gabriel, AAC1

import processing.serial.*;
import java.util.StringTokenizer;

/////////////////////////////////////////////////////////////////////////////
// Constants zone

// These constants will update currentMode variable, which will show the current graph mode (one from 'All sensors', 'Light sensor', 'Temp sensor', 'Humid sensor')
final int temperatureSensor = 0;
final int lightSensor = 1;
final int humiditySensor = 2;
final int allSensors = 3;

// Text which will appear inside of each button. I had to hack this a little bit by adding some spaces for better fitting
final String buttonLabels[] = {
  "        Start      ", "      Node 1      ", "      Node 2      ", "      Node 3      ", "  Temp Sensor", "  Light Sensor", "  Humid Sensor", "    All Sensors"
};

// Text which appears as an identifier for each graph
final String graphLabels[] = {
  "Temp", "Light", "Humid"
};

// Max Temp value, Max Light value, Max Humid value
final int maxDisplayedValues[] = {
  100, 255, 255
};

// Text sizes
final int buttonLabelTextSize = 15;
final int gridsLabelTextSize = 10;

// Height and width of the sketch
final int w = 1360;
final int h = 710;

// The space on X axis between 0 and the start of the buttons column
final int xButtonStart = w / 100;

// The space on Y axis between 0 and the start of the first button
final int yButtonStart = h / 50;

// Button sizes
final int buttonHeight = (h - 2 * yButtonStart) / 14;
final int buttonWidth = w / 12;

// Space between buttons, on Y axis
final int spaceBetweenButtons = (h - 2 * yButtonStart - 8 * buttonHeight) / 7;

// Space on X axis between two consecutive points in a plot
// This basically gives the plot speed, because i plot (graphWidth / xDif) points at the same time.
final int xDif = 10;

/////////////////////////////////////////////////////////////////////////////
// Variables zone

// Start by default with node 1
int selectedNode = 1;
// Start by default with all sensors
int currentMode = allSensors;
// Start by default with startDisabled (you must press the Start button) 
int started = 0;
// I will assume that the gatway-node is connected at Serial.list()[0] COM port
Serial myPort;
// Here is where the values received from the nodes will be kept
float[][] tempVals;
float[][] lightVals;
float[][] humidVals;
int[] startPoints;

/////////////////////////////////////////////////////////////////////////////
// Functions. Let's play... It's an ugly code, I must admit...

void allocArrays() {
  int sensorGraphWidth = (width - 3 * xButtonStart - 2 * buttonWidth);
  
  tempVals = new float[3][sensorGraphWidth / xDif + 1];
  lightVals = new float[3][sensorGraphWidth / xDif + 1];
  humidVals = new float[3][sensorGraphWidth / xDif + 1];
  startPoints = new int[3];
  startPoints[0] = sensorGraphWidth / xDif + 1;
  startPoints[1] = sensorGraphWidth / xDif + 1;
  startPoints[2] = sensorGraphWidth / xDif + 1;
}

void drawButtons() {
  stroke(0);
  strokeWeight(2);
  
  // Draw each button
  for (int i = 0; i < 8; i++)
  {
    // If the Start button was pressed
    if (started == 1)
    {
      // Fill all currently selected buttons with yellow
      // (we must have at least three yellow buttons at any time:
      //  one representing currently selected button, one representing currently selected graph mode and last, but not least, the Start button)  
      if ((i == 0 && started == 1) || (i == 1 && selectedNode == 1) || (i == 2 && selectedNode == 2) || 
          (i == 3 && selectedNode == 3) || (i == 4 && currentMode == 0) || (i == 5 && currentMode == 1) ||
          (i == 6 && currentMode == 2) || (i == 7 && currentMode == 3))
        fill(#FFFF00);
      else
        // Boring gray unselected buttons
        fill(#CCCECC);
    }
    else
      // If Start button was not pressed, all buttons are boring and gray and unselected
      fill(#CCCECC);    
      
    rect(xButtonStart, yButtonStart + buttonHeight * i + spaceBetweenButtons * i, buttonWidth, buttonHeight);
    fill(#090A09);
    textSize(buttonLabelTextSize);
    text(buttonLabels[i], xButtonStart, yButtonStart + buttonHeight * i + spaceBetweenButtons * i + buttonLabelTextSize + (buttonHeight - buttonLabelTextSize) / 2);
  }
}

void drawLines() {
  int sensorGraphHeight = (height - 2 * yButtonStart) / 3;
  int sensorGraphWidth = (width - 3 * xButtonStart - 2 * buttonWidth);
  stroke(0);
  strokeWeight(2);
  
  // || ... vertical left and right lines
  line(2 * xButtonStart + 2 * buttonWidth, yButtonStart, 2 * xButtonStart + 2 * buttonWidth, height - yButtonStart);
  line(width - xButtonStart, yButtonStart, width - xButtonStart, height - yButtonStart);
  
  // = ... horizontal top and bottom lines
  line(2 * xButtonStart + 2 * buttonWidth, yButtonStart, width - xButtonStart, yButtonStart);
  line(2 * xButtonStart + 2 * buttonWidth, height - yButtonStart, width - xButtonStart, height - yButtonStart);

  if (currentMode == allSensors)
  {
    // Draw two more lines to create three graph zones (we currently have just one big rectangular, created from the 4 lines drawn above)
    line(2 * xButtonStart + 2 * buttonWidth, yButtonStart + sensorGraphHeight, width - xButtonStart, yButtonStart + sensorGraphHeight);
    line(2 * xButtonStart + 2 * buttonWidth, yButtonStart + 2 * sensorGraphHeight, width - xButtonStart, yButtonStart + 2 * sensorGraphHeight);
  
    // Put some text at the left of each graph zone, as an identifier: Temp, Light, Humid
    textSize(buttonLabelTextSize);
    text(graphLabels[0], 2 * xButtonStart + buttonWidth * 4 / 3, yButtonStart + (sensorGraphHeight - buttonLabelTextSize) / 2 + buttonLabelTextSize);
    text(graphLabels[1], 2 * xButtonStart + buttonWidth * 4 / 3, yButtonStart + sensorGraphHeight + (sensorGraphHeight - buttonLabelTextSize) / 2 + buttonLabelTextSize);
    text(graphLabels[2], 2 * xButtonStart + buttonWidth * 4 / 3, yButtonStart + 2 * sensorGraphHeight + (sensorGraphHeight - buttonLabelTextSize) / 2 + buttonLabelTextSize);
    
    strokeWeight(0.1);
    textSize(gridsLabelTextSize);
    // Draw value lines in each graph zone, in order to see better the sensor value
    // Lines from 10 degrees to ten degrees for temp zone
    // Lines from (int) (255 / 10) to (int) (255 / 10) for Light and Humid 
    for (int i = 0; i < 9; i++) {
      for (int j = 1; j < 4; j++) {
        int heightInGraphZone = (i + 1) * ( sensorGraphHeight / 10 );
        line(2 * xButtonStart + 2 * buttonWidth, yButtonStart + sensorGraphHeight * j - heightInGraphZone - 4, width - xButtonStart, yButtonStart + sensorGraphHeight * j - heightInGraphZone - 4);
        text((int)((maxDisplayedValues[j-1] / 10.0) * (i + 1)) + ""  , 2 * xButtonStart + 2 * buttonWidth - 20, yButtonStart + sensorGraphHeight * j - heightInGraphZone - 4);
      }
    }
  } else {
    textSize(buttonLabelTextSize);
    text(graphLabels[currentMode], 2 * xButtonStart + buttonWidth * 4 / 3, yButtonStart + sensorGraphHeight  + (sensorGraphHeight - buttonLabelTextSize) / 2 + buttonLabelTextSize);
    
    strokeWeight(0.1);
    textSize(gridsLabelTextSize);
    // Draw value lines in each graph zone, in order to see better the sensor value
    // Lines from (int) (100 / 40) degrees to (int) (100 / 40) degrees for temp
    // Lines from (int) (255 / 40) to (int) (255 / 40) for Light and Humid 
    for (int i = 0; i < 39; i++) {
      int heightInGraphZone = (i + 1) * ( (height - 2 * yButtonStart) / 40 );
      line(2 * xButtonStart + 2 * buttonWidth, height - yButtonStart - heightInGraphZone, width - xButtonStart, height - yButtonStart - heightInGraphZone);
      text((int)((maxDisplayedValues[currentMode] / 40.0) * (i + 1)) + ""  , 2 * xButtonStart + 2 * buttonWidth - 20, height - yButtonStart - heightInGraphZone);
    }
  }
}

void setup() {
  // I will assume that the gatway-node is connected at Serial.list()[0] COM port
  myPort = new Serial(this, Serial.list()[0], 9600);
  // don't generate a serialEvent() unless you get a newline character:
  myPort.bufferUntil('\n');

  size(w, h);
  smooth();
  
  background(200);

  // Alloc arrays
  allocArrays();

  //draw Buttons
  drawButtons();

  // Draw Lines
  drawLines();
}


void draw() {
  // do nothing. Everything happens in SerialEvent 
}

void redrawSketch(int nodeId) {
  background(200);
  //draw Buttons
  drawButtons();

  // Draw Lines
  drawLines();

  if (currentMode == allSensors) {
    // Draw lines connecting all points
    for (int i = startPoints[nodeId - 1]; i < tempVals[0].length - 1; i++) {
      strokeWeight(2);
    
      int sensorGraphWidth = (width - 3 * xButtonStart - 2 * buttonWidth);
      int sensorGraphHeight = (height - 2 * yButtonStart) / 3;
      
      int x = i * xDif + 2 * xButtonStart + 2 * buttonWidth + 2;
      
      float tempY1height = (tempVals[nodeId - 1][i] * (sensorGraphHeight - 4)) / 100 + 2;
      float tempY2height = (tempVals[nodeId - 1][i+1] * (sensorGraphHeight - 4)) / 100 + 2;
      float tempY1 = yButtonStart + sensorGraphHeight - tempY1height;
      float tempY2 = yButtonStart + sensorGraphHeight - tempY2height;
      
      float lightY1height = (lightVals[nodeId - 1][i] * (sensorGraphHeight - 4)) / 255 + 2;
      float lightY2height = (lightVals[nodeId - 1][i+1] * (sensorGraphHeight - 4)) / 255 + 2;
      float lightY1 = yButtonStart + sensorGraphHeight * 2 - lightY1height;
      float lightY2 = yButtonStart + sensorGraphHeight * 2 - lightY2height;
      
      float humidY1height = (humidVals[nodeId - 1][i] * (sensorGraphHeight - 4)) / 255 + 2;
      float humidY2height = (humidVals[nodeId - 1][i+1] * (sensorGraphHeight - 4)) / 255 + 2;
      float humidY1 = yButtonStart + sensorGraphHeight * 3 - humidY1height;
      float humidY2 = yButtonStart + sensorGraphHeight * 3 - humidY2height;
      
      stroke(255, 0, 0);
      line(x,tempY1,x+xDif,tempY2);
      stroke(0, 255, 0);
      line(x,lightY1,x+xDif,lightY2);
      stroke(0, 0, 255);
      line(x,humidY1,x+xDif,humidY2);
    }
  } else {
    // Draw lines connecting all points
    for (int i = startPoints[nodeId - 1]; i < tempVals[0].length - 1; i++) {
      strokeWeight(2);
    
      int sensorGraphWidth = (width - 3 * xButtonStart - 2 * buttonWidth);
      int sensorGraphHeight = height - 2 * yButtonStart;
      
      int x = i * xDif + 2 * xButtonStart + 2 * buttonWidth + 2;
      
      if (currentMode == temperatureSensor)
      {
        float tempY1height = (tempVals[nodeId - 1][i] * (sensorGraphHeight - 4)) / 100 + 2;
        float tempY2height = (tempVals[nodeId - 1][i+1] * (sensorGraphHeight - 4)) / 100 + 2;
        float tempY1 = yButtonStart + sensorGraphHeight - tempY1height;
        float tempY2 = yButtonStart + sensorGraphHeight - tempY2height;
        stroke(255, 0, 0);
        line(x,tempY1,x+xDif,tempY2);
      }
      
      if (currentMode == lightSensor)
      {
        float lightY1height = (lightVals[nodeId - 1][i] * (sensorGraphHeight - 4)) / 255 + 2;
        float lightY2height = (lightVals[nodeId - 1][i+1] * (sensorGraphHeight - 4)) / 255 + 2;
        float lightY1 = yButtonStart + sensorGraphHeight - lightY1height;
        float lightY2 = yButtonStart + sensorGraphHeight - lightY2height;
        stroke(0, 255, 0);
        line(x,lightY1,x+xDif,lightY2);
      }
      
      if (currentMode == humiditySensor)
      {
        float humidY1height = (humidVals[nodeId - 1][i] * (sensorGraphHeight - 4)) / 255 + 2;
        float humidY2height = (humidVals[nodeId - 1][i+1] * (sensorGraphHeight - 4)) / 255 + 2;
        float humidY1 = yButtonStart + sensorGraphHeight - humidY1height;
        float humidY2 = yButtonStart + sensorGraphHeight - humidY2height;
        stroke(0, 0, 255);
        line(x,humidY1,x+xDif,humidY2);
      }
    }
  }
}

void mousePressed() {
  int buttonsPressed[] = {0, 0, 0, 0, 0, 0, 0, 0};
  int buttonWasPressed = 0;

  if (mouseButton == LEFT)
  {
    for (int i = 0; i < 8; i++)
    {
      if ((mouseX > xButtonStart) && (mouseX < xButtonStart + buttonWidth) &&
        (mouseY > yButtonStart + buttonHeight * i + spaceBetweenButtons * i) && (mouseY < yButtonStart + buttonHeight * i + spaceBetweenButtons * i + buttonHeight))
      {
        buttonsPressed[i] = 1;
        buttonWasPressed = 1;
      }
    }
    
    if (buttonsPressed[0] == 1)
      started = 1;
    if (buttonsPressed[1] == 1)
      selectedNode = 1;
    if (buttonsPressed[2] == 1)
      selectedNode = 2;
    if (buttonsPressed[3] == 1)
      selectedNode = 3;
    if (buttonsPressed[4] == 1)
      currentMode = temperatureSensor;
    if (buttonsPressed[5] == 1)
      currentMode = lightSensor;
    if (buttonsPressed[6] == 1)
      currentMode = humiditySensor;
    if (buttonsPressed[7] == 1)
      currentMode = allSensors;
      
    if (buttonWasPressed == 1)
    {
      redrawSketch(selectedNode);
    }
  }
}

void serialEvent (Serial myPort) {
  if (started == 0)
    return;
  
  if (myPort.available() <= 0)
    return; 
  
  // get the ASCII string:
  String inString = null;

  byte tempBuffer[] = myPort.readBytesUntil('\n');
  if (tempBuffer == null) {
  } 
  else {
    inString = new String(tempBuffer);
  }

  if (inString != null) {
    // trim off any whitespace:
    inString = trim(inString);
    
    StringTokenizer st = new StringTokenizer(inString, ":");
    int nodeId = Integer.parseInt((String)st.nextElement());
    int temp = Integer.parseInt((String)st.nextElement());
    int light = Integer.parseInt((String)st.nextElement());
    int humid = Integer.parseInt((String)st.nextElement());
    
    System.out.println(nodeId + " " + temp + " " + light + " " + humid);
    // Filter spikes
    if ((nodeId == 1 || nodeId == 2 || nodeId == 3) &&
        (temp >= 0 && temp <= 100) &&
        (light >=0 && light <= 255) &&
        (humid >=0 && humid <=255))
    {
      if (startPoints[nodeId - 1] > 0)
        startPoints[nodeId - 1]--;
      
      tempVals[nodeId - 1][tempVals[0].length - 1] = temp;
      lightVals[nodeId - 1][tempVals[0].length - 1] = light;
      humidVals[nodeId - 1][tempVals[0].length - 1] = humid;
    
      // Redraw the sketc only if we received a package for a currently selected node
      // Otherwise keep the values to be drawn later
      if (nodeId == selectedNode)
        redrawSketch(nodeId);
    
      // Slide everything down in the array
      for (int i = 0; i < tempVals[0].length - 1; i++) {
        tempVals[nodeId - 1][i] = tempVals[nodeId - 1][i + 1];
        lightVals[nodeId - 1][i] = lightVals[nodeId - 1][i + 1];
        humidVals[nodeId - 1][i] = humidVals[nodeId - 1][i + 1];
      }
    }
  }
}

